@@ -52,6 +52,7 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #ifdef HAVE_ALLOCA_H
 #include <alloca.h>
 #endif
+#include <stdatomic.h>
 
 #include "misc.h"
 #include "pcscd.h"
@@ -107,9 +108,7 @@ LONG _RefReader(READER_CONTEXT * sReader)
 	if (0 == sReader->reference)
 		return SCARD_E_READER_UNAVAILABLE;
 
-	pthread_mutex_lock(&sReader->reference_lock);
 	sReader->reference += 1;
-	pthread_mutex_unlock(&sReader->reference_lock);
 
 	return SCARD_S_SUCCESS;
 }
@@ -119,9 +118,7 @@ LONG _UnrefReader(READER_CONTEXT * sReader)
 	if (0 == sReader->reference)
 		return SCARD_E_READER_UNAVAILABLE;
 
-	pthread_mutex_lock(&sReader->reference_lock);
 	sReader->reference -= 1;
-	pthread_mutex_unlock(&sReader->reference_lock);
 
 	if (0 == sReader->reference)
 		removeReader(sReader);
@@ -141,6 +138,9 @@ LONG RFAllocateReaderSpace(unsigned int customMaxReaderHandles)
 	{
 		sReadersContexts[i] = malloc(sizeof(READER_CONTEXT));
 		sReadersContexts[i]->vHandle = NULL;
+		atomic_init(&sReadersContexts[i]->hLockId, 0);
+		atomic_init(&sReadersContexts[i]->contexts, 0);
+		atomic_init(&sReadersContexts[i]->reference, 0);
 
 		/* Zero out each value in the struct */
 		memset(readerStates[i].readerName, 0, MAX_READERNAME);
@@ -311,8 +311,6 @@ LONG RFAddReader(const char *readerNameLong, int port, const char *library,
 	sReadersContexts[dwContext]->powerState = POWER_STATE_UNPOWERED;
 
 	/* reference count */
-	(void)pthread_mutex_init(&sReadersContexts[dwContext]->reference_lock,
-		NULL);
 	sReadersContexts[dwContext]->reference = 1;
 
 	/* If a clone to this reader exists take some values from that clone */
@@ -514,8 +512,6 @@ LONG RFAddReader(const char *readerNameLong, int port, const char *library,
 		sReadersContexts[dwContextB]->powerState = POWER_STATE_UNPOWERED;
 
 		/* reference count */
-		(void)pthread_mutex_init(&sReadersContexts[dwContextB]->reference_lock,
-			NULL);
 		sReadersContexts[dwContextB]->reference = 1;
 
 		/* Call on the parent driver to see if the slots are thread safe */
@@ -623,7 +619,7 @@ LONG RFRemoveReader(const char *readerName, int port, int flags)
 					if ((IFD_SUCCESS) == ret && (1 == tagValue[0]))
 					{
 						tagValue[0] = 1;
-						ret = IFDSetCapabilities(sReadersContexts[i],
+						IFDSetCapabilities(sReadersContexts[i],
 							TAG_IFD_DEVICE_REMOVED, sizeof tagValue, tagValue);
 					}
 				}
@@ -1190,7 +1186,7 @@ SCARDHANDLE RFCreateReaderHandle(READER_CONTEXT * rContext)
 		/* FIXME: THIS IS NOT STRONG ENOUGH: A 128-bit token should be
 		 * generated.  The client and server would associate token and hCard
 		 * for authentication. */
-		randHandle = SYS_RandomInt(0, -1);
+		randHandle = SYS_RandomInt();
 
 		/* do we already use this hCard somewhere? */
 		ret = RFReaderInfoById(randHandle, &dummy_reader);
@@ -1232,7 +1228,7 @@ LONG RFAddReaderHandle(READER_CONTEXT * rContext, SCARDHANDLE hCard)
 	}
 
 	newHandle->hCard = hCard;
-	newHandle->dwEventStatus = 0;
+	atomic_init(&newHandle->dwEventStatus, 0);
 
 	lrv = list_append(&rContext->handlesList, newHandle);
 	if (lrv < 0)
@@ -1313,6 +1309,7 @@ LONG RFCheckReaderEventState(READER_CONTEXT * rContext, SCARDHANDLE hCard)
 {
 	LONG rv;
 	RDR_CLIHANDLES *currentHandle;
+	DWORD dwEventStatus;
 
 	(void)pthread_mutex_lock(&rContext->handlesList_lock);
 	currentHandle = list_seek(&rContext->handlesList, &hCard);
@@ -1324,7 +1321,8 @@ LONG RFCheckReaderEventState(READER_CONTEXT * rContext, SCARDHANDLE hCard)
 		return SCARD_E_INVALID_HANDLE;
 	}
 
-	switch(currentHandle->dwEventStatus)
+	dwEventStatus = currentHandle->dwEventStatus;
+	switch(dwEventStatus)
 	{
 		case 0:
 			rv = SCARD_S_SUCCESS;
@@ -1398,11 +1396,10 @@ void RFCleanupReaders(void)
 
 			if (rv != SCARD_S_SUCCESS)
 				Log2(PCSC_LOG_ERROR, "RFRemoveReader error: 0x%08lX", rv);
-
-			free(sReadersContexts[i]);
-
-			sReadersContexts[i] = NULL;
 		}
+
+		free(sReadersContexts[i]);
+		sReadersContexts[i] = NULL;
 	}
 
 #ifdef USE_SERIAL
