@@ -210,7 +210,7 @@ LONG SCardEstablishContext(DWORD dwScope, /*@unused@*/ LPCVOID pvReserved1,
 	 * identified by clients and distinguished from others
 	 */
 
-	*phContext = SYS_RandomInt(0, -1);
+	*phContext = SYS_RandomInt();
 
 	Log2(PCSC_LOG_DEBUG, "Establishing Context: 0x%lX", *phContext);
 
@@ -715,10 +715,8 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			}
 
 			/* the card is now in use */
-			(void)pthread_mutex_lock(&rContext->powerState_lock);
-			rContext->powerState = POWER_STATE_IN_USE;
+			RFSetPowerState(rContext, POWER_STATE_IN_USE);
 			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_IN_USE");
-			(void)pthread_mutex_unlock(&rContext->powerState_lock);
 		}
 	}
 
@@ -900,7 +898,7 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 			/* SCARD_UNPOWER_CARD */
 			rv = IFDPowerICC(rContext, IFD_POWER_DOWN, NULL, NULL);
 
-			rContext->powerState = POWER_STATE_UNPOWERED;
+			RFSetPowerState(rContext, POWER_STATE_UNPOWERED);
 			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_UNPOWERED");
 		}
 
